; Add this file to the end of your .emacs file or load this file from your .emacs file.
; On a PC, place the .emacs file in c:/.
; On unix, put the .emacs file in the root of your home directory.

; loads the LAML emacs support
(load "@laml-dir@laml.el") 

; loads the Scheme Elucidator Emacs support.
; Consider autoload instead.
(load "@laml-dir@styles/elucidator/elucidator.el")  

; loads the LENO emacs support.
; Consider autoload instead.
(load "@laml-dir@styles/lecture-notes/leno.el")

; Customize LAML in a new frame.
(defun laml-customize ()
  (interactive)
  (let ((new-frame (make-frame)))
    (select-frame new-frame)
    (customize-group 'laml)))

; -----------------------------------------------------------------------------
; Laml mode key bindings.

; Arrange that C-o starts asynchronous laml processing
(define-key laml-mode-map "\C-o" 'laml-process-current-buffer)

; Arrange that C-x C-o starts synchronous laml processing
(define-key laml-mode-map "\C-x\C-o" 'laml-sync-process-current-buffer)

; Arrange that C-x C-p makes laml-output vissible.
(define-key laml-mode-map "\C-x\C-p" 'see-laml-output)

; Arrange that C-x C-n calls the LAML nest command:
(define-key laml-mode-map "\C-x\C-n" 'nest)

; Arrange that C-x C-m calls the LAML unnest command:
(define-key laml-mode-map "\C-x\C-m" 'unnest)

; Arrange that C-x C-e calls the LAML embed command:
(define-key laml-mode-map "\C-x\C-e" 'embed)

; Arrange that C-x C-r calls the LAML unembed command:
(define-key laml-mode-map "\C-x\C-r" 'unembed)

; Arrange that C-x C-q calls the LAML split command:
(define-key laml-mode-map "\C-x\C-q" 'split)

; Arrange that C-x C-a calls the LAML unsplit command:
(define-key laml-mode-map "\C-x\C-a" 'unsplit)

; Arrange that C-M-q indents the currently selected S-expression
(define-key laml-mode-map "\C-\M-q" 'indent-sexp)

; Arrange that TAB (C-i) runs indent-for-tab-command, as in Scheme mode.
(define-key laml-mode-map "\C-i" 'indent-for-tab-command)

; Arrange that C-M-x activates lisp-eval-defun (definition af Lisp expression in inferior Lisp job).
(define-key laml-mode-map "\C-\M-x" 'lisp-eval-defun)

; An alais to lisp-eval-defun
(define-key laml-mode-map "\C-c\C-e" 'lisp-eval-defun)

; Arrange that C-c C-r activates lisp-eval-region (definition af Lisp expressions in inferior Lisp job).
(define-key laml-mode-map "\C-c\C-r" 'lisp-eval-region)

(define-key laml-mode-map "\C-i" 'goto-next-input-field)

; -----------------------------------------------------------------------------
; Laml mode menu bindings.

(define-key laml-mode-map [menu-bar laml]
    (cons "Laml" (make-sparse-keymap "Laml")))


; -----------------------------------------------------------------------------
; Templates:

(define-key laml-mode-map [menu-bar laml laml-insert-template]
    (cons "Insert LAML template" (make-sparse-keymap "laml-templates")))

(define-key laml-mode-map [menu-bar laml laml-insert-template other]
      '("Prompt me..." . laml-insert-template))



(define-key laml-mode-map [menu-bar laml laml-insert-template xhtml1-frameset-page]
      '("XHTML1.0 frameset page" . (lambda () (interactive) (laml-insert-template "xhtml1-frameset-page"))))

(define-key laml-mode-map [menu-bar laml laml-insert-template xhtml1-transitional-page]
      '("XHTML1.0 transitional page" . (lambda () (interactive) (laml-insert-template "xhtml1-transitional-page"))))

(define-key laml-mode-map [menu-bar laml laml-insert-template xhtml1-strict-page]
      '("XHTML1.0 strict page" . (lambda () (interactive) (laml-insert-template "xhtml1-strict-page"))))

(define-key laml-mode-map [menu-bar laml laml-insert-template html401-page]
      '("HTML4.01 transitional page" . (lambda () (interactive) (laml-insert-template "html401-page"))))



(define-key laml-mode-map [menu-bar laml menu-forms-sep3]
    '("----"))

; -----------------------------------------------------------------------------
; Scheme evaluation

(define-key laml-mode-map [menu-bar laml scheme-eval]
     (cons "Evaluate Scheme form(s)" (make-sparse-keymap "Scheme evaluation")))

(define-key laml-mode-map [menu-bar laml scheme-eval lisp-eval-buffer]
      '("Entire buffer" . (lambda () (interactive) (lisp-eval-region (point-min) (point-max)))))

(define-key laml-mode-map [menu-bar laml scheme-eval lisp-eval-region]
      '("Current selection" . lisp-eval-region))

(define-key laml-mode-map [menu-bar laml scheme-eval lisp-eval-defun]
      '("Current form" . lisp-eval-defun))

; -----------------------------------------------------------------------------


(define-key laml-mode-map [menu-bar laml indent-sexp]
      '("Indent symbolic expression" . indent-sexp))

; -----------------------------------------------------------------------------

(define-key laml-mode-map [menu-bar laml menu-forms-sep2]
    '("----"))

(define-key laml-mode-map [menu-bar laml unnest]
      '("Unnest" . unnest))

(define-key laml-mode-map [menu-bar laml nest]
      '("Nest form in form" . nest))

(define-key laml-mode-map [menu-bar laml unsplit]
      '("Unsplit" . unsplit))

(define-key laml-mode-map [menu-bar laml split]
      '("Split string" . split))

(define-key laml-mode-map [menu-bar laml unembed]
      '("Unembed" . unembed))

(define-key laml-mode-map [menu-bar laml embed]
      '("Embed string in form" . embed))

(define-key laml-mode-map [menu-bar laml menu-forms-sep1]
    '("----"))

(define-key laml-mode-map [menu-bar laml process-sync]
      '("Process syncronously" . laml-sync-process-current-buffer))

(define-key laml-mode-map [menu-bar laml process-async]
      '("Process asyncronously" . laml-process-current-buffer))

; End main LAML menu entries


; -----------------------------------------------------------------------------
; Scheme evaluation

; Loads Emacs Lisp Scheme stuff
(scheme-mode)

(defun lisp-eval-buffer () 
  (interactive)
  (lisp-eval-region (point-min) (point-max)))

(define-key scheme-mode-map "\C-\M-x" 'lisp-eval-defun)
(define-key scheme-mode-map "\C-\M-y" 'lisp-eval-region)
(define-key scheme-mode-map "\C-\M-z" 'lisp-eval-buffer)

(define-key scheme-mode-map [menu-bar scheme menu-forms-sep-scheme]
    '("----"))

(define-key scheme-mode-map [menu-bar scheme scheme-eval]
      (cons "Evaluate Scheme form(s)" (make-sparse-keymap "Scheme evaluation")))
 
(define-key scheme-mode-map [menu-bar scheme scheme-eval lisp-eval-buffer]
      '("Entire buffer" . lisp-eval-buffer))

(define-key scheme-mode-map [menu-bar scheme scheme-eval lisp-eval-region]
      '("Current selection" . lisp-eval-region))

(define-key scheme-mode-map [menu-bar scheme scheme-eval lisp-eval-defun]
      '("Current form" . lisp-eval-defun))


; -----------------------------------------------------------------------------
; Contributions to the Emacs tools menu:


(define-key global-map [menu-bar tools menu-forms-sep-tools]
    '("----"))

(define-key global-map [menu-bar tools laml]
    (cons "Laml" (make-sparse-keymap "Laml")))

; -------------------------------------------------------------------------------
; Elucidator bindings the tools>laml>elucidator-commands

(define-key global-map [menu-bar tools laml elucidator]
     (cons "Elucidator commands" (make-sparse-keymap "Elucidator")))

(define-key global-map [menu-bar tools laml elucidator quit-elucidator]
    '("Quit Elucidator" . quit-elucidator))

(define-key global-map [menu-bar tools laml elucidator elucidator-help]
    '("Elucidator help" . elucidator-help))

(define-key global-map [menu-bar tools laml elucidator elucidator-status]
    '("Elucidator status" . elucidator-status))

(define-key global-map [menu-bar tools laml elucidator menu-forms-sep-tools-elu-3]
    '("----"))



(define-key global-map [menu-bar tools laml elucidator elucidator-let-be]
    '("Elucidator let be..." . elucidator-let-be))

(define-key global-map [menu-bar tools laml elucidator large-program-window]
    '("Large program window" . large-program-window))

(define-key global-map [menu-bar tools laml elucidator large-documentation-window]
    '("Large documentation window" . large-documentation-window))


(define-key global-map [menu-bar tools laml elucidator back]
    '("Go back" . back))

(define-key global-map [menu-bar tools laml elucidator goto]
    '("Goto" . goto))


(define-key global-map [menu-bar tools laml elucidator surround-none-ref]
    '("Surround prog ref - none" . surround-none-ref))

(define-key global-map [menu-bar tools laml elucidator surround-prog-ref]
    '("Surround prog ref" . surround-prog-ref))





(define-key global-map [menu-bar tools laml elucidator show-setup]
    '("Show setup" . show-setup))

(define-key global-map [menu-bar tools laml elucidator show-program]
    '("Show program" . show-program))

(define-key global-map [menu-bar tools laml elucidator elucidator-show-processing]
    '("Elucidator show processing" . elucidator-show-processing))



(define-key global-map [menu-bar tools laml elucidator doc-ref]
    '("Insert doc ref" . doc-ref))

(define-key global-map [menu-bar tools laml elucidator prog-ref]
    '("Insert prog ref" . prog-ref))

(define-key global-map [menu-bar tools laml elucidator insert-sectional-comment]
    '("Insert sectional comment..." . insert-sectional-comment))



(define-key global-map [menu-bar tools laml elucidator menu-forms-sep-tools-elu-2]
    '("----"))


(define-key global-map [menu-bar tools laml elucidator reset-elucidator]
    '("Reset elucidator" . reset-elucidator))

(define-key global-map [menu-bar tools laml elucidator refresh-elucidator]
    '("Refresh elucidator" . refresh-elucidator))

(define-key global-map [menu-bar tools laml elucidator menu-forms-sep-tools-elu-1]
    '("----"))

(define-key global-map [menu-bar tools laml elucidator elucidator-insert-documentation-entry]
    '("Insert documentation entry" . elucidator-insert-documentation-entry))

(define-key global-map [menu-bar tools laml elucidator elucidator-insert-documentation-section]
    '("Insert documentation section" . elucidator-insert-documentation-section))


(define-key global-map [menu-bar tools laml elucidator menu-forms-sep-tools-elu-0]
    '("----"))

(define-key global-map [menu-bar tools laml elucidator elucidate]
   '("Elucidative processing" . elucidate))



; End Elucidator bindings


(define-key global-map [menu-bar tools laml setup-elucidator]
    '("Setup existing elucidator..." . setup-elucidator))

(define-key global-map [menu-bar tools laml make-elucidator]
    '("Make new elucidator..." . make-elucidator))

(define-key global-map [menu-bar tools laml make-lecture-notes]
  '("Make new LENO lecture notes..." . make-lecture-notes))

(define-key global-map [menu-bar tools laml laml-customize]
      '("Customize LAML" . laml-customize))

(define-key global-map [menu-bar tools laml set-interactive-laml-mirror-library]
      '("Set mirror of LAML interaction..." . set-interactive-laml-mirror-library))

(define-key global-map [menu-bar tools laml run-laml-interactively]
      '("Run LAML interactively" . run-laml-interactively))




